/*
* Copyright (c) Innosilicon Technology Ltd. All Rights Reserved
* Dual MIT/GPLv2
*
* The contents of this file are subject to the MIT license as set out below.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Alternatively, the contents of this file may be used under the terms of
* the GNU General Public License Version 2 ("GPL") in which case the provisions
* of GPL are applicable instead of those above.
*
* If you wish to allow use of your version of this file only under the terms of
* GPL, and not to allow others to use your version of this file under the terms
* of the MIT license, indicate your decision by deleting the provisions above
* and replace them with the notice and other provisions required by GPL as set
* out in the file called "GPL-COPYING" included in this distribution. If you do
* not delete the provisions above, a recipient may use your version of this file
* under the terms of either the MIT license or GPL.
*
* This License is also included in this distribution in the file called
* "MIT-COPYING".
*
* EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
* PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#ifndef __INNO_DRM_H__
#define __INNO_DRM_H__

#include <linux/types.h>

typedef void inno_drm_file;
typedef void inno_drm_device;
typedef void inno_drm_scdc;
typedef void inno_drm_connector;
typedef void inno_edid;
typedef void inno_drm_crtc;
typedef void inno_drm_plane;
typedef void inno_drm_plane_state;
typedef void inno_drm_framebuffer;
typedef void inno_drm_format_info;
typedef void inno_drm_crtc_state;
typedef void inno_drm_connector_state;
typedef void inno_videomode;
typedef void inno_drm_mode_object;
typedef void inno_drm_gem_object;
typedef void inno_drm_dp_aux;
typedef void inno_drm_dp_link;

struct inno_drm_format_name_buf{
	char str[32];
};



void *inno_get_drm_file_prvdata(inno_drm_file *file);
void inno_set_drm_file_prvdata(inno_drm_file *file, void *data);

bool inno_is_support_scdc(inno_drm_scdc *scdc);
bool inno_connector_is_support_scdc(inno_drm_connector *connector);

ssize_t inno_drm_scdc_read(void *adapter, u8 offset, void *buffer,
		      size_t size);
ssize_t inno_drm_scdc_write(void *adapter, u8 offset,
		       const void *buffer, size_t size);
inno_edid *inno_drm_do_get_edid(inno_drm_connector *connector,
	int (*get_edid_block)(void *data, u8 *buf, unsigned int block,
			      size_t len), void *data);
int inno_drm_edid_header_is_valid(const u8 *raw_edid);
int inno_drm_edid_block_checksum(const u8 *raw_edid);
int inno_drm_do_probe_ddc_edid(void *data, u8 *buf, unsigned int block, size_t len);
bool inno_drm_irq_enabled(inno_drm_device *drm_dev);
bool inno_drm_crtc_handle_vblank(inno_drm_crtc *crtc);
void *inno_drm_dev_get_dev(inno_drm_device *drm_dev);
void *inno_drm_dev_get_prvdata(inno_drm_device *drm_dev);
bool inno_dp_drm_helper_hpd_irq_event(inno_drm_device *dev);


inno_drm_device *inno_drm_plane_get_drmdev_dev(inno_drm_plane *plane);
inno_drm_device *inno_drm_plane_get_drmdev(inno_drm_plane *plane);
char *inno_drm_plane_get_name(inno_drm_plane *plane);
void *inno_drm_plane_get_drmdev_private(inno_drm_plane *plane);
inno_drm_plane_state *inno_drm_plane_get_state(inno_drm_plane *plane);
#define inno_drm_plane_get_member(member, plane) inno_drm_plane_get_##member(plane)


unsigned int inno_drm_plane_state_get_rotation(inno_drm_plane_state *state);
unsigned int inno_drm_plane_state_get_src_w(inno_drm_plane_state *state);
unsigned int inno_drm_plane_state_get_src_h(inno_drm_plane_state *state);
unsigned int inno_drm_plane_state_get_src_x(inno_drm_plane_state *state);
unsigned int inno_drm_plane_state_get_src_y(inno_drm_plane_state *state);
unsigned int inno_drm_plane_state_get_crtc_w(inno_drm_plane_state *state);
unsigned int inno_drm_plane_state_get_crtc_h(inno_drm_plane_state *state);
unsigned int inno_drm_plane_state_get_crtc_x(inno_drm_plane_state *state);
unsigned int inno_drm_plane_state_get_crtc_y(inno_drm_plane_state *state);
void *inno_drm_plane_state_get_fb(inno_drm_plane_state *state);
void *inno_drm_plane_state_get_crtc(inno_drm_plane_state *state);
#define inno_drm_plane_state_get_member(member, state) inno_drm_plane_state_get_##member(state)



unsigned int *inno_drm_fb_get_pitches(inno_drm_framebuffer *fb);
unsigned int *inno_drm_fb_get_offsets(inno_drm_framebuffer *fb);
const inno_drm_format_info *inno_drm_fb_get_format_info(inno_drm_framebuffer *fb);
#define inno_drm_fb_get_member(member, fb) inno_drm_fb_get_##member(fb)




unsigned int inno_drm_format_info_get_format(const inno_drm_format_info *format);
unsigned char *inno_drm_format_info_get_cpp(const inno_drm_format_info *format);
#define inno_drm_format_info_get_member(member, format) inno_drm_format_info_get_##member(format)



void *inno_drm_crtc_get_state(inno_drm_crtc *crtc);
void *inno_drm_crtc_get_name(inno_drm_crtc *crtc);
void *inno_drm_crtc_get_dev(inno_drm_crtc *crtc);
#define inno_drm_crtc_get_member(member, crtc) inno_drm_crtc_get_##member(crtc)


void *inno_drm_crtc_state_get_adjusted_mode(inno_drm_crtc_state *state);
void *inno_drm_crtc_state_get_ctm(inno_drm_crtc_state *state);
void *inno_drm_crtc_state_get_gamma_lut(inno_drm_crtc_state *state);
bool inno_drm_crtc_state_get_color_mgmt_changed(inno_drm_crtc_state *state);
unsigned int inno_drm_crtc_state_get_ctm_baseid(inno_drm_crtc_state *state);
unsigned int inno_drm_crtc_state_get_gamma_baseid(inno_drm_crtc_state *state);
bool inno_drm_crtc_state_get_active(inno_drm_crtc_state *state);
#define inno_drm_crtc_state_get_member(member, crtc) inno_drm_crtc_state_get_##member(crtc)



inno_videomode *inno_drm_videomode_alloc(void);
void inno_drm_videomode_free(inno_videomode *mode);
int inno_drm_videomode_get_hactive(inno_videomode *mode);
unsigned int inno_drm_videomode_get_hfront_porch(inno_videomode *mode);
unsigned int inno_drm_videomode_get_hback_porch(inno_videomode *mode);
unsigned int inno_drm_videomode_get_hsync_len(inno_videomode *mode);
int inno_drm_videomode_get_vactive(inno_videomode *mode);
unsigned int inno_drm_videomode_get_vfront_porch(inno_videomode *mode);
unsigned int inno_drm_videomode_get_vback_porch(inno_videomode *mode);
unsigned int inno_drm_videomode_get_vsync_len(inno_videomode *mode);
unsigned int inno_drm_videomode_get_flags(inno_videomode *mode);
unsigned long inno_drm_videomode_get_pixelclock(inno_videomode *mode);
#define inno_drm_videomode_get_member(member, mode) inno_drm_videomode_get_##member(mode)

void inno_drm_get_format_name(u32 format, struct inno_drm_format_name_buf format_name);


u32 INNO_DRM_FORMAT_ARGB2101010_FUNC(void);
u32 INNO_DRM_FORMAT_ABGR2101010_FUNC(void);
u32 INNO_DRM_FORMAT_RGBA1010102_FUNC(void);
u32 INNO_DRM_FORMAT_BGRA1010102_FUNC(void);
u32 INNO_DRM_FORMAT_ARGB8888_FUNC(void);
u32 INNO_DRM_FORMAT_ABGR8888_FUNC(void);
u32 INNO_DRM_FORMAT_RGBA8888_FUNC(void);
u32 INNO_DRM_FORMAT_BGRA8888_FUNC(void);
u32 INNO_DRM_FORMAT_XRGB8888_FUNC(void);
u32 INNO_DRM_FORMAT_XBGR8888_FUNC(void);
u32 INNO_DRM_FORMAT_RGBX8888_FUNC(void);
u32 INNO_DRM_FORMAT_BGRX8888_FUNC(void);
u32 INNO_DRM_FORMAT_RGB888_FUNC(void);
u32 INNO_DRM_FORMAT_BGR888_FUNC(void);
u32 INNO_DRM_FORMAT_RGBA5551_FUNC(void);
u32 INNO_DRM_FORMAT_ABGR1555_FUNC(void);
u32 INNO_DRM_FORMAT_RGB565_FUNC(void);
u32 INNO_DRM_FORMAT_BGR565_FUNC(void);
u32 INNO_DRM_FORMAT_UYVY_FUNC(void);
u32 INNO_DRM_FORMAT_YUYV_FUNC(void);
u32 INNO_DRM_FORMAT_NV12_FUNC(void);
u32 INNO_DRM_FORMAT_YUV420_FUNC(void);



#define INNO_DRM_FORMAT_ARGB2101010    INNO_DRM_FORMAT_ARGB2101010_FUNC()
#define INNO_DRM_FORMAT_ABGR2101010    INNO_DRM_FORMAT_ABGR2101010_FUNC()
#define INNO_DRM_FORMAT_RGBA1010102    INNO_DRM_FORMAT_RGBA1010102_FUNC()
#define INNO_DRM_FORMAT_BGRA1010102     INNO_DRM_FORMAT_BGRA1010102_FUNC()
#define INNO_DRM_FORMAT_ARGB8888        INNO_DRM_FORMAT_ARGB8888_FUNC()
#define INNO_DRM_FORMAT_ABGR8888        INNO_DRM_FORMAT_ABGR8888_FUNC()
#define INNO_DRM_FORMAT_RGBA8888       INNO_DRM_FORMAT_RGBA8888_FUNC()
#define INNO_DRM_FORMAT_BGRA8888       INNO_DRM_FORMAT_BGRA8888_FUNC()
#define INNO_DRM_FORMAT_XRGB8888        INNO_DRM_FORMAT_XRGB8888_FUNC()
#define INNO_DRM_FORMAT_XBGR8888        INNO_DRM_FORMAT_XBGR8888_FUNC()
#define INNO_DRM_FORMAT_RGBX8888        INNO_DRM_FORMAT_RGBX8888_FUNC()
#define INNO_DRM_FORMAT_BGRX8888       INNO_DRM_FORMAT_BGRX8888_FUNC()
#define INNO_DRM_FORMAT_RGB888          INNO_DRM_FORMAT_RGB888_FUNC()
#define INNO_DRM_FORMAT_BGR888         INNO_DRM_FORMAT_BGR888_FUNC()
#define INNO_DRM_FORMAT_RGBA5551        INNO_DRM_FORMAT_RGBA5551_FUNC()
#define INNO_DRM_FORMAT_ABGR1555        INNO_DRM_FORMAT_ABGR1555_FUNC()
#define INNO_DRM_FORMAT_RGB565         INNO_DRM_FORMAT_RGB565_FUNC()
#define INNO_DRM_FORMAT_BGR565          INNO_DRM_FORMAT_BGR565_FUNC()
#define INNO_DRM_FORMAT_UYVY         INNO_DRM_FORMAT_UYVY_FUNC()
#define INNO_DRM_FORMAT_YUYV          INNO_DRM_FORMAT_YUYV_FUNC()
#define INNO_DRM_FORMAT_NV12         INNO_DRM_FORMAT_NV12_FUNC()
#define INNO_DRM_FORMAT_YUV420          INNO_DRM_FORMAT_YUV420_FUNC()



u32 INNO_DISPLAY_FLAGS_HSYNC_HIGH_FUNC(void);
u32 INNO_DISPLAY_FLAGS_VSYNC_HIGH_FUNC(void);
u32 INNO_DISPLAY_FLAGS_INTERLACED_FUNC(void);


#define INNO_DISPLAY_FLAGS_HSYNC_HIGH   INNO_DISPLAY_FLAGS_HSYNC_HIGH_FUNC()
#define INNO_DISPLAY_FLAGS_VSYNC_HIGH   INNO_DISPLAY_FLAGS_VSYNC_HIGH_FUNC()
#define INNO_DISPLAY_FLAGS_INTERLACED   INNO_DISPLAY_FLAGS_INTERLACED_FUNC()

u32 connector_status_connected_func(void);
u32 connector_status_disconnected_func(void);
u32 connector_status_unknown_func(void);
#define inno_connector_status_unknown		connector_status_unknown_func()
#define inno_connector_status_connected		connector_status_connected_func()
#define inno_connector_status_disconnected	connector_status_disconnected_func()
const char *inno_drm_get_connector_status_name(int);

u32 INNO_DP_AUX_NATIVE_REPLY_ACK_FUNC(void);
u32 INNO_DP_AUX_NATIVE_REPLY_NACK_FUNC(void);
u32 INNO_DP_AUX_NATIVE_REPLY_DEFER_FUNC(void);
u32 INNO_DP_AUX_NATIVE_REPLY_MASK_FUNC(void);
#define INNO_DP_AUX_NATIVE_REPLY_ACK     INNO_DP_AUX_NATIVE_REPLY_ACK_FUNC()
#define INNO_DP_AUX_NATIVE_REPLY_NACK    INNO_DP_AUX_NATIVE_REPLY_NACK_FUNC()
#define INNO_DP_AUX_NATIVE_REPLY_DEFER   INNO_DP_AUX_NATIVE_REPLY_DEFER_FUNC()
#define INNO_DP_AUX_NATIVE_REPLY_MASK    INNO_DP_AUX_NATIVE_REPLY_MASK_FUNC()


u32  INNO_DRM_MODE_ROTATE_MASK_FUNC(void);
u32  INNO_DRM_MODE_ROTATE_0_FUNC(void);
u32  INNO_DRM_MODE_ROTATE_90_FUNC(void);
u32  INNO_DRM_MODE_ROTATE_180_FUNC(void);
u32  INNO_DRM_MODE_ROTATE_270_FUNC(void);
#define INNO_DRM_MODE_ROTATE_MASK		INNO_DRM_MODE_ROTATE_MASK_FUNC()
#define INNO_DRM_MODE_ROTATE_0			INNO_DRM_MODE_ROTATE_0_FUNC()
#define INNO_DRM_MODE_ROTATE_90			INNO_DRM_MODE_ROTATE_90_FUNC()
#define INNO_DRM_MODE_ROTATE_180		INNO_DRM_MODE_ROTATE_180_FUNC()
#define INNO_DRM_MODE_ROTATE_270		INNO_DRM_MODE_ROTATE_270_FUNC()

u32  INNO_DRM_MODE_REFLECT_X_FUNC(void);
u32  INNO_DRM_MODE_REFLECT_Y_FUNC(void);
#define INNO_DRM_MODE_REFLECT_X			INNO_DRM_MODE_REFLECT_X_FUNC()
#define INNO_DRM_MODE_REFLECT_Y			INNO_DRM_MODE_REFLECT_Y_FUNC()


inno_drm_gem_object *inno_drm_gem_object_lookup(inno_drm_file * filp, uint32_t handle);
void inno_drm_gem_object_put(void *gem_obj);


#endif
