#include <linux/i2c.h>
#include <linux/io.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include "innopmbus_drv.h"
#include "hal_interface.h"

#include <linux/cpufreq.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/device.h>
#include <linux/version.h>
#include <linux/thermal.h>

#include "inno_timer.h"
#include "inno_io.h"
#include "hal.h"
#include "innopower.h"
#include "inno_devfreq_gov.h"

#define INNO_POWERCHIP_MAX_NUM	32

static struct i2c_device_id inno_powerchip_id_tbl[INNO_POWERCHIP_MAX_NUM];
static int id_tbl_idx = 0;
static struct i2c_driver *i2c_drv = NULL;

#define HAL_DRV_MCUFW_STAT_VOL_TO_MCU  (0x1 << 0)
#define HAL_DRV_MCUFW_STAT_VOL_TO_HOST (0x1 << 1)
#define HAL_DRV_MCUFW_STAT_VOL_DILE 0

enum HAL_DRV_MCUFW_CMD_VOL {
	HAL_DRV_MCUFW_CMD_VOL_SET = 1,
	HAL_DRV_MCUFW_CMD_VOL_GET,
	HAL_DRV_MCUFW_CMD_VOL_MAX,
};

struct vol_cmd_param {
	uint32_t mv;

	uint8_t scene; /*set cmd*/
};

#define HAL_DRV_MCUFW_MODULE_VOL_OFFSET 4

#define INNOPWR_IF_ERR_MSG(error, mask) if ((error) & (mask)) innopwr_notice("vol cmd failed and error msg is:[%s]\n", #mask);
enum DRV_MCUFW_PWR_ERR_MSG {
	PWR_ERR_MSG_REQUESTING = (0x1<<0),
	PWR_ERR_MSG_OPERATE_DONE = (0x1<<1),
	PWR_ERR_MSG_OPERATE_FAILED = (0x1<<2),
	PWR_ERR_MSG_OPERATE_BUSY = (0x1<<3),
	PWR_ERR_MSG_CRC_CKECK_FAILE = (0x1<<4),
	PWR_ERR_MSG_CMD_ERR = (0x1<<5),
	PWR_ERR_MSG_PMBUS_RESET = (0x1<<6),
	PWR_ERR_MSG_HAL_RUN_FAILED = (0x1<<7),
	PWR_ERR_MSG_PMBUS_ADDR_ERR = (0x1<<8),
	PWR_ERR_MSG_PMBUS_CMD_EXE_ERR = (0x1<<9),
	PWR_ERR_MSG_TIMOUT_MACHINE_START_STATE = (0x1<<10),
	PWR_ERR_MSG_TIMOUT_MACHINE_OTHER_STATE = (0x1<<11),
	PWR_ERR_MSG_TIMOUT_MACHINE_CLKSTRETCH = (0x1<<12),
	PWR_ERR_MSG_PMBUS_NACK = (0x1<<13),
	PWR_ERR_MSG_WR_FIFO_UNDERFLOW = (0x1<<14),
	PWR_ERR_MSG_RD_FIFO_OVERFLOW = (0x1<<15),
	PWR_ERR_MSG_MAX,
};

static void innopwr_err_msg_dump(uint32_t error)
{
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_REQUESTING);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_OPERATE_DONE);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_OPERATE_FAILED);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_OPERATE_BUSY);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_CRC_CKECK_FAILE);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_CMD_ERR);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_PMBUS_RESET);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_HAL_RUN_FAILED);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_PMBUS_ADDR_ERR);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_PMBUS_CMD_EXE_ERR);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_TIMOUT_MACHINE_START_STATE);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_TIMOUT_MACHINE_OTHER_STATE);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_TIMOUT_MACHINE_CLKSTRETCH);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_PMBUS_NACK);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_WR_FIFO_UNDERFLOW);
	INNOPWR_IF_ERR_MSG(error, PWR_ERR_MSG_RD_FIFO_OVERFLOW);
}

int innopwr_drv_mcufw_vol_recv_msg(void *chip_ctx, uint32_t *msg, uint32_t cnt)
{
	int i = 0;
	unsigned int temp_data = 0;
	struct dev_rsrc *pdev_rsrc = (struct dev_rsrc *)chip_ctx;
	void __iomem* vol_vram_base = pdev_rsrc->drv_mcufw_vram + HAL_DRV_MCUFW_MODULE_VOL_OFFSET;

	/* 拷贝数据     inno_memcpy_fromio函数再x86_64 环境不同机器上测试会报错, mttr设置时会报内存属性错误 */
	for (i = 0; i < cnt; i++ ) {
		msg[i] = 0;
		temp_data = inno_ioread32(vol_vram_base + i*4);
		memcpy(msg + i, &temp_data, 4);
		if (get_pwr_debug_lvl() >= PWRD_LVL_INFO) {
			innopwr_notice("vram_addr=0x%p temp_data is %u msg[%d]=%u\n", vol_vram_base + i*4, temp_data, i, msg[i]);
		}
	}

	return 0;
}

int innopwr_drv_mcufw_vol_send_msg(void *chip_ctx, uint32_t *msg, uint32_t cnt)
{
	struct dev_rsrc *pdev_rsrc = (struct dev_rsrc *)chip_ctx;
	void __iomem* vol_vram_base = pdev_rsrc->drv_mcufw_vram + HAL_DRV_MCUFW_MODULE_VOL_OFFSET;
	int i = 0;

	for (i = 0; i < cnt; i++) inno_iowrite32(msg[i], vol_vram_base+i*4);

	return 0;
}

int innopwr_drv_mcufw_vol_clear_status(void *chip_ctx)
{
	uint8_t module_raw[HAL_DRV_MCUFW_MODULE_LEN] = {0};
	uint32_t *msg = (uint32_t *)module_raw;
	struct dev_rsrc *pdev_rsrc = (struct dev_rsrc *)chip_ctx;

	msg[0] = 0;
	msg[1] = HAL_DRV_MCUFW_STAT_VOL_DILE;
	msg[2] = 0;
	msg[3] = 0;

	innopwr_drv_mcufw_vol_send_msg(pdev_rsrc, (uint32_t *)msg, 8);
	if (get_pwr_debug_lvl() >= PWRD_LVL_INFO) {
		innopwr_notice("\n------------------------------------------------->\n");
		innopwr_drv_mcufw_vol_recv_msg(chip_ctx, msg, 8);
		innopwr_notice("\n<-------------------------------------------------\n");
	}
	return 0;
}

int innopwr_drv_mcufw_vol_cmd(void *chip_ctx, enum HAL_DRV_MCUFW_CMD_VOL cmd, struct vol_cmd_param *pparam)
{
	int retry = 10;
	uint8_t module_raw[HAL_DRV_MCUFW_MODULE_LEN] = {0};
	uint32_t *msg = (uint32_t *)module_raw;
	char *ptemp = NULL;
	struct dev_rsrc *pdev_rsrc = (struct dev_rsrc *)chip_ctx;
	bool wait_idle = false;
	bool mcufw_complete = false;
	uint32_t crc = 0;
	uint32_t target_mv = pparam->mv;

	if (!(HAL_DRV_MCUFW_CMD_VOL_SET == cmd || HAL_DRV_MCUFW_CMD_VOL_GET == cmd)) {
		innopwr_notice("cmd[%d] is unkown\n", cmd);
		return -1;
	}

	/*wait idle status*/
	mutex_lock(pdev_rsrc->vol_mutex);
	do {
		innopwr_drv_mcufw_vol_recv_msg(chip_ctx, msg, 8);
		if (msg[1] == HAL_DRV_MCUFW_STAT_VOL_DILE) {
			/*send get voltage cmd*/
			wait_idle = true;
			break;
		}
		retry--;
		inno_msleep(1);
	} while (retry >= 0);

	/*check idle*/
	if (wait_idle) {
		/*status*/
		msg[1] = HAL_DRV_MCUFW_STAT_VOL_TO_MCU;
		/*cmd*/
		msg[2] = cmd;
		/*cmd params*/
		if (cmd == HAL_DRV_MCUFW_CMD_VOL_SET) {
			msg[3] = target_mv;

			/*protocol version2: add the case param between kmddrv and mcufw*/
			ptemp = ((char *)(&msg[4]));
			*ptemp = pparam->scene;
		} else {
			msg[3] = 0;
		}

		/*crc calculate*/
		msg[0] = hal_drv_mcufw_crc32_le(crc, (unsigned char *)&msg[1], HAL_DRV_MCUFW_MODULE_LEN-4);

		innopwr_drv_mcufw_vol_send_msg(pdev_rsrc, (uint32_t *)msg, 8);

		/*make sure send msg successful*/
		/*
		 * barrier()
		 */

		hal_drv_mcufw_irq_trigger(chip_ctx);
		if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) {
			innopwr_notice("send Vol cmd[%d %s] done msg[0-4]=0x%x 0x%x 0x%x 0x%x 0x%x\n",
				cmd, cmd == HAL_DRV_MCUFW_CMD_VOL_SET ? "Set" : "Get", msg[0], msg[1], msg[2], msg[3], msg[4]);
		}
	} else {
		innopwr_notice("wait idle is failed\n");
		goto cmd_failed;
	}

	/*loop mcufw complete statue*/
	cmd == HAL_DRV_MCUFW_CMD_VOL_SET ? retry = 50 : 10;
	do {
		innopwr_drv_mcufw_vol_recv_msg(chip_ctx, msg, 8);
		if (msg[1] == HAL_DRV_MCUFW_STAT_VOL_TO_HOST) {
			if (msg[0] ==  hal_drv_mcufw_crc32_le(crc, (unsigned char *)&msg[1], HAL_DRV_MCUFW_MODULE_LEN-4)) {
				mcufw_complete = true;
				if (cmd == HAL_DRV_MCUFW_CMD_VOL_GET) {
					pparam->mv = msg[3]; /*out param*/
					if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) {
						innopwr_notice("get mv retry = %d mv is msg[3] = %u done\n", retry, msg[3]);
					}
				};
				break;
			}
		}
		inno_msleep(2);
		retry--;
	} while(retry >= 0);

	if (!mcufw_complete) {
		innopwr_notice("Vol cmd %s timeout error code:[%u]\n", cmd == HAL_DRV_MCUFW_CMD_VOL_SET ? "Set" : "Get", msg[1]);
		innopwr_err_msg_dump(msg[1]);
		goto cmd_failed;
	}
	innopwr_drv_mcufw_vol_clear_status(chip_ctx);
	mutex_unlock(pdev_rsrc->vol_mutex);
	if (get_pwr_debug_lvl() >= PWRD_LVL_NOTICE) {
		innopwr_notice("retry[%d], Vol cmd[%d, %s] successful *mv = %u target_mv = %u\n", retry, cmd, cmd == HAL_DRV_MCUFW_CMD_VOL_SET ? "Set" : "Get", pparam->mv, target_mv);
	}

	return 0;
cmd_failed:
	innopwr_drv_mcufw_vol_clear_status(chip_ctx);
	mutex_unlock(pdev_rsrc->vol_mutex);

	return -1;

}

int innopwr_drv_mcufw_set_voltage(void* chip_ctx, uint32_t mv, uint8_t scene)
{
	struct vol_cmd_param param;

	param.mv = mv;
	param.scene = scene;
	return innopwr_drv_mcufw_vol_cmd(chip_ctx, HAL_DRV_MCUFW_CMD_VOL_SET, &param);
}

int innopwr_drv_mcufw_get_voltage_from_bmc(void* chip_ctx, uint32_t *mv)
{
	return hal_get_voltage_from_bmc(chip_ctx, mv);
}

int innopwr_drv_mcufw_get_voltage_from_reg(void* chip_ctx, uint32_t *mv)
{
	struct vol_cmd_param param;

	param.mv = *mv;
	param.scene = PWR_NOTIFY_FOR_NORMAL;

	/*update param to take param mv*/
	if (innopwr_drv_mcufw_vol_cmd(chip_ctx, HAL_DRV_MCUFW_CMD_VOL_GET, &param)) {
		innopwr_notice("cmd HAL_DRV_MCUFW_CMD_VOL_GET failed and origiin mv is %u\n", *mv);
		return -1;
	}
	*mv = param.mv;

	return 0;
}

int innopwr_drv_mcufw_get_voltage(void* chip_ctx, uint32_t *mv)
{
	if (PWRD_DBG_REAL_VOL == get_pwr_debug_lvl()) {
		return innopwr_drv_mcufw_get_voltage_from_reg(chip_ctx, mv);
	} else {
		return innopwr_drv_mcufw_get_voltage_from_bmc(chip_ctx, mv);
	}
}

int innopower_set_voltage(void *chip_ctx, unsigned int vol, uint8_t scene)
{
	struct dev_rsrc *pdev_rsrc = (struct dev_rsrc *)chip_ctx;
	struct innopower *power = platform_get_drvdata(pdev_rsrc->power_dev[0]);

	if (!is_support_update_voltage()) {
		if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) {
			innopwr_notice("is not support_update_voltage and return");
		}
		return 0;
	}

	if (!hal_vol_is_digital(pdev_rsrc->dev)) {
		return INNO_POWER_OK;
	}

	if (!pdev_rsrc->drv_mcufw_v) {
		if (!power || !power->chip || !power->chip->set_gpu_voltage) {
			innopwr_notice("null pointer\n");
			return INNO_POWER_ERR;
		}

		return power->chip->set_gpu_voltage(power->chip, 0, vol);
	} else {
		return innopwr_drv_mcufw_set_voltage(chip_ctx, vol, scene);
	}
}

int innopower_get_voltage(void *chip_ctx)
{
	struct dev_rsrc *pdev_rsrc = (struct dev_rsrc *)chip_ctx;
	struct innopower *power = platform_get_drvdata(pdev_rsrc->power_dev[0]);
	int vol = 0;

	if (!hal_vol_is_digital(pdev_rsrc->dev)) {
		return ANALOG_VOLT_DEFAULT_OUTPUT;
	}

	if (!pdev_rsrc->drv_mcufw_v) {
		if (!power || !power->chip || !power->chip->get_gpu_voltage) {
			innopwr_notice("null pointer\n");
			return INNO_POWER_ERR;
		}

		vol = power->chip->get_gpu_voltage(power->chip, 0);
		return vol;
	} else {
		if (!innopwr_drv_mcufw_get_voltage(chip_ctx, (uint32_t *)&vol)) {
			innopwr_info("get mv = %d\n", vol);
			return vol;
		} else {
			innopwr_notice("get failed\n");
			return INNO_POWER_ERR;
		}
	}
}

ssize_t innopower_store_gpu_voltage(struct device *dev, struct device_attribute *attr, const char *buf, size_t count)
{
	int retval = 0;
	int gpu_voltage = 0;
	struct innopower *power = platform_get_drvdata(to_platform_device(dev));
	struct dev_rsrc *pdev_rsrc = inno_rsrc_devres_find(power->pdev);

	if (!is_support_update_voltage()) {
		return count;
	}

	if (sscanf(buf, " %i ", &gpu_voltage) != 1) {
		innopwr_err("Failed to store  gpu_voltage attribute\n");
		return -EINVAL;
	}

	if (!pdev_rsrc->drv_mcufw_v) {
		if (!power || !power->chip || !power->chip->set_gpu_voltage) {
			innopwr_notice("null pointer\n");
			return -EINVAL;
		}

		retval = power->chip->set_gpu_voltage(power->chip, 0, gpu_voltage);
	} else {
			hal_set_voltage(power->pdev, (unsigned int)gpu_voltage);
	}
	innopwr_notice("gpu_voltage = %d\n", gpu_voltage);

	return retval ? retval : count;
}

ssize_t innopower_show_gpu_voltage(struct device *dev, struct device_attribute *attr, char *buf)
{
	int  retval = 0;
	int  gpu_voltage = 0;
	int  retry = 0;
	struct innopower *power = platform_get_drvdata(to_platform_device(dev));
	struct dev_rsrc *pdev_rsrc = inno_rsrc_devres_find(power->pdev);

	do {
		retval = innopower_get_voltage(pdev_rsrc);
		retry++;
		inno_msleep(5);
	} while ((retry < 5) && (retval <= 0 || retval > 1200));

	if (retval <= 0 || retval > 1200) {
		innopwr_err("Failed to read  gpu_voltage attribute\n");
		sprintf(buf, "0\n");
		return retval;
	}
	gpu_voltage = retval;
	innopwr_notice("[%s]: gpu_voltage = %d retry=[%d]\n",
		gpu_voltage > 950 ? "warnning" : "success", retval, retry);

	return sprintf(buf, "%d\n", gpu_voltage);
}

DEVICE_ATTR(gpu_voltage, S_IRUGO | S_IWUSR, innopower_show_gpu_voltage, innopower_store_gpu_voltage);

static int innopwr_pmbus_rs_lock(struct powerchip *pwrchip, bool chip_need_rs)
{
	struct inno_pmbus_dev *pmbus = NULL;
	pmbus = (struct inno_pmbus_dev *)pwrchip->priv_data;
	if (!pmbus) {
		innopwr_err("pmbus get failed!!!\n");
		return INNO_POWER_ERR;
	}
	mutex_lock(&pmbus->chip_rs_lock);
	pmbus->chip_need_rs = chip_need_rs;
	return INNO_POWER_OK;
}

static int innopwr_pmbus_rs_unlock(struct powerchip *pwrchip)
{
	struct inno_pmbus_dev *pmbus = NULL;
	pmbus = (struct inno_pmbus_dev *)pwrchip->priv_data;
	if (!pmbus) {
		innopwr_err("pmbus get failed!!!\n");
		return INNO_POWER_ERR;
	}
	pmbus->chip_need_rs = 1;
	mutex_unlock(&pmbus->chip_rs_lock);
	return INNO_POWER_OK;
}

static int _innopwr_read8(struct powerchip *pwrchip, unsigned char reg, bool chip_need_rs)
{
	int ret = 0;
	if (!pwrchip) {
		innopwr_notice("gchip is null!!!\n");
		return INNO_POWER_ERR;
	}

	mutex_lock(&pwrchip->lock);
	if (innopwr_pmbus_rs_lock(pwrchip, chip_need_rs)) {
		mutex_unlock(&pwrchip->lock);
		innopwr_notice("%s read reg[0x%x] pmbus_rs_lock failed!!!\n", pwrchip->name, reg);
		return INNO_POWER_ERR;
	}

	ret = i2c_smbus_read_byte_data(pwrchip->client, reg);
	if (ret < 0) {
		innopwr_pmbus_rs_unlock(pwrchip);
		mutex_unlock(&pwrchip->lock);
		innopwr_err("%s read reg[0x%x] failed!!!\n", pwrchip->name, reg);

		return INNO_POWER_ERR;
	}
	innopwr_pmbus_rs_unlock(pwrchip);
	mutex_unlock(&pwrchip->lock);

	return ret;
}

static int _innopwr_write8(struct powerchip *pwrchip, unsigned char reg, unsigned char data, bool chip_need_rs)
{
	int ret = 0;

	if (!pwrchip) {
		innopwr_notice("gchip is null!!!\n");
		return INNO_POWER_ERR;
	}

	mutex_lock(&pwrchip->lock);
	if (innopwr_pmbus_rs_lock(pwrchip, chip_need_rs)) {
		mutex_unlock(&pwrchip->lock);
		innopwr_notice("%s write 0x%x to reg[0x%x] pmbus_rs_lock failed!!!\n", pwrchip->name, data, reg);
		return INNO_POWER_ERR;
	}
	ret = i2c_smbus_write_byte_data(pwrchip->client, reg, data);
	if (ret < 0) {
		innopwr_pmbus_rs_unlock(pwrchip);
		mutex_unlock(&pwrchip->lock);
		innopwr_err("%s write 0x%x to reg[0x%x] failed!!!\n", pwrchip->name, data, reg);

		return INNO_POWER_ERR;
	}
	innopwr_pmbus_rs_unlock(pwrchip);
	mutex_unlock(&pwrchip->lock);

	return INNO_POWER_OK;
}

static int _innopwr_read16(struct powerchip *pwrchip, unsigned char reg, bool chip_need_rs)
{
	int ret = 0;

	if (!pwrchip) {
		innopwr_notice("gchip is null!!!\n");
		return INNO_POWER_ERR;
	}

	mutex_lock(&pwrchip->lock);
	if (innopwr_pmbus_rs_lock(pwrchip, chip_need_rs)) {
		mutex_unlock(&pwrchip->lock);
		innopwr_notice("%s read reg[0x%x] pmbus_rs_lock failed!!!\n", pwrchip->name, reg);
		return INNO_POWER_ERR;
	}
	ret = i2c_smbus_read_word_data(pwrchip->client, reg);
	if (ret < 0) {
		innopwr_pmbus_rs_unlock(pwrchip);
		mutex_unlock(&pwrchip->lock);
		innopwr_err("%s read reg[0x%x] failed!!!\n", pwrchip->name, reg);

		return INNO_POWER_ERR;
	}
	innopwr_pmbus_rs_unlock(pwrchip);
	mutex_unlock(&pwrchip->lock);

	return ret;
}

static int _innopwr_write16(struct powerchip *pwrchip, unsigned char reg, unsigned int data, bool chip_need_rs)
{
	int ret = 0;

	if (!pwrchip) {
		innopwr_notice("gchip is null!!!\n");
		return INNO_POWER_ERR;
	}

	mutex_lock(&pwrchip->lock);
	if (innopwr_pmbus_rs_lock(pwrchip, chip_need_rs)) {
		mutex_unlock(&pwrchip->lock);
		innopwr_notice("%s write 0x%x to reg[0x%x] pmbus_rs_lock failed!!!\n", pwrchip->name, data, reg);
		return INNO_POWER_ERR;
	}
	ret = i2c_smbus_write_word_data(pwrchip->client, reg, data);
	if (ret < 0) {
		innopwr_pmbus_rs_unlock(pwrchip);
		mutex_unlock(&pwrchip->lock);
		innopwr_err("%s write 0x%x to reg[0x%x] failed!!!\n", pwrchip->name, data, reg);

		return INNO_POWER_ERR;
	}
	innopwr_pmbus_rs_unlock(pwrchip);
	mutex_unlock(&pwrchip->lock);

	return ret;
}

int innopwr_read8(struct powerchip *pwrchip, unsigned char reg)
{
	return _innopwr_read8(pwrchip, reg, 1);
}

int innopwr_read8_no_repeat_start(struct powerchip *pwrchip, unsigned char reg)
{
	return _innopwr_read8(pwrchip, reg, 0);
}

int innopwr_write8(struct powerchip *pwrchip, unsigned char reg, unsigned char data)
{
	return _innopwr_write8(pwrchip, reg, data, 1);
}

int innopwr_write8_no_repeat_start(struct powerchip *pwrchip, unsigned char reg, unsigned char data)
{
	return _innopwr_write8(pwrchip, reg, data, 0);
}

int innopwr_read16(struct powerchip *pwrchip, unsigned char reg)
{
	return _innopwr_read16(pwrchip, reg, 1);
}

int innopwr_read16_no_repeat_start(struct powerchip *pwrchip, unsigned char reg)
{
	return _innopwr_read16(pwrchip, reg, 0);
}

int innopwr_write16(struct powerchip *pwrchip, unsigned char reg, unsigned int data)
{
	return _innopwr_write16(pwrchip, reg, data, 1);
}

int innopwr_write16_no_repeat_start(struct powerchip *pwrchip, unsigned char reg, unsigned int data)
{
	return _innopwr_write16(pwrchip, reg, data, 0);
}

int innopower_sys_register(struct device *dev)
{
	return device_create_file(dev, &dev_attr_gpu_voltage);
}

void innopower_sys_unregister(struct device *dev)
{
	device_remove_file(dev, &dev_attr_gpu_voltage);
}

#define I2C_SLAVE_ADDR_VALID 0x7F
#define I2C_SLAVE_ADDR_INVALID 0x0
#define I2C_SLAVE_ADDE_DEFAULT_VALID 0x20
/*default double chip use xdpe12284c*/
#define DOUBLE_CHIP_IDX0_DEFAULT_ADDR 0xEC
int innopower_register_chip(struct innopower *power)
{
	struct powerchip *chip;
	struct i2c_board_info info;

	chip = devm_kzalloc(power->pltdev, sizeof(struct powerchip), GFP_KERNEL);
	if (!chip) {
		innopwr_err("devm_kzalloc failed\n");
		goto chip_fail;
	}

	/*i2c info static register*/
	/*double is not support hwinfo and custom info parase, and have a default addr*/

	memset(&info, 0, sizeof(struct i2c_board_info));
	if (power->params->slave_addr == I2C_SLAVE_ADDR_INVALID) {
		power->params->slave_addr = DOUBLE_CHIP_IDX0_DEFAULT_ADDR ;
		if (power->params->slave_addr > I2C_SLAVE_ADDR_VALID) {
			info.addr = power->params->slave_addr >> 1;
		} else {
			info.addr = power->params->slave_addr;
		}

		strcpy(power->params->chip_name, "VIRTURE_CHIP");
	}

	info.addr = power->params->slave_addr;
	if (info.addr > I2C_SLAVE_ADDR_VALID) { /*default xdpe12284c*/
		info.addr = DOUBLE_CHIP_IDX0_DEFAULT_ADDR >> 1;
	}
	sprintf(info.type, "%s-%d", power->params->chip_name, id_tbl_idx);

#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 2, 0))
	chip->client = i2c_new_device(power->adapter, &info);
#else
	chip->client = i2c_new_client_device(power->adapter, &info);
#endif
	if (!chip->client) {
		innopwr_err("i2c_new_device failed\n");
		goto chip_fail;
	}
	i2c_set_clientdata(chip->client, chip);
	power->chip = chip;
	chip->priv_data = i2c_get_adapdata(power->adapter);

	/*inno_powerchip_id_tbl is created dynamic*/
	strcpy(inno_powerchip_id_tbl[id_tbl_idx].name, chip->client->name);
	id_tbl_idx++;

	i2c_drv = innopower_get_drvobj(power->params->chip_name);
	if (!i2c_drv) {
		innopwr_err("not support drv\n");
		goto chip_fail;
	}
	innopwr_info("name = %s type= %s addr = 0x%x i2c_drv = %px\n", chip->client->name, info.type, info.addr, i2c_drv);

	return 0;

chip_fail:
	devm_kfree(power->pltdev, chip);
	power->chip = NULL;

	return -ENOMEM;
}

int innopower_unregister_chip(struct innopower *power)
{
	if (power->chip) {
		i2c_unregister_device(power->chip->client);
		devm_kfree(power->pltdev, power->chip);
	}

	return 0;
}

static uint32_t innopwr_find_vol(inno_dev *dev, uint32_t gpu_freq)
{
	int i = 0;
	uint32_t vol = 0;
	struct inno_gpu_opp *opp_tbl = hal_power_get_opptbl(dev);
	int32_t opp_size = hal_power_get_opptbl_size(dev);

	for (i = 0; i < opp_size; i++) {
		if (gpu_freq == opp_tbl[i].freq / GPU_FREQ_M2HZ_UNIT) {
			vol = opp_tbl[i].volt / GPU_VOL_UV2MV_UNIT;
			break;
		}
	}

	if (!vol) {
		vol = opp_tbl[opp_size-1].volt / GPU_VOL_UV2MV_UNIT;
		innopwr_notice("found init vol failed and take max vol[%u] as init vol opp_size[%d]\n", vol, opp_size);
	}

	return vol;
}

static int innopower_vol_init(inno_dev *dev, uint8_t scene)
{
	struct dev_rsrc *pdev_rsrc = inno_rsrc_devres_find(dev);
	struct power_custom_hwinfo_t *pwrinfo = &pdev_rsrc->pwrinfo;
	uint8_t  pwr_default_mode = pwrinfo->pwr_ctrl_mode;
	uint32_t init_vol = 0;
	uint16_t init_freq = 0;

	if (!is_support_update_voltage()) {
		return 0;
	}

	switch (pwr_default_mode) {
		case DEFAULT_WORKMODE_POWSAVE:
			init_freq = pwrinfo->powersave_mode_freq;
			break;
		case DEFAULT_WORKMODE_DYN:
			init_freq = pwrinfo->dyn_mode_mode_freq;
			break;
		case DEFAULT_WORKMODE_PERF:
			init_freq = pwrinfo->performance_mode_freq;
			break;
		case DEFAULT_WORKMODE_NORMAL:
		default:
			init_freq = pwrinfo->normal_mode_freq;
			break;
	}

	init_vol = innopwr_find_vol(dev, init_freq);
	if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) {
		innopwr_notice("pwr_default_mode = %u init_freq[%d] init_vol[%u]\n", pwr_default_mode, init_freq, init_vol);
	}

	if (INNO_POWER_OK != innopower_set_voltage(pdev_rsrc, init_vol, scene)) {
		innopwr_err("set hw init voltage failed and pwr_default_mode=[%u] init_freq=[%d] init_vol[%u]\n", pwr_default_mode, init_freq, init_vol);
		return INNO_POWER_ERR;
	}

	return INNO_POWER_OK;
}

static int innopower_pmbus_probe(struct platform_device *pdev)
{
	struct innopower *power;
	struct device *dev = &pdev->dev;
	struct dev_rsrc *pdev_rsrc = NULL;
	int init_vol = 0;

	power = devm_kzalloc(dev, sizeof(*power), GFP_KERNEL);
	if (!power) {
		innopwr_err("devm_kzalloc failed\n");
		goto power_fail;
	}

	power->pltdev = dev;
	platform_set_drvdata(pdev, power);
	power->params = hal_get_volctrl_hwinfo(pdev->dev.parent);
	power->adapter = hal_get_pmbus_adapter(pdev->dev.parent, power->params->pmbus_id);
	if (!power->adapter) {
		innopwr_err("i2c_get_adapter failed\n");
		goto power_fail;
	}

	innopower_register_chip(power);

	power->pdev = pdev->dev.parent;
	pdev_rsrc = inno_rsrc_devres_find(power->pdev);
	init_vol = innopower_get_voltage(pdev_rsrc);
	hal_set_voltage_ops(power->pdev, innopower_set_voltage, innopower_get_voltage);
	innopower_vol_init(power->pdev, PWR_NOTIFY_FOR_NORMAL);
	innopower_sys_register(&pdev->dev);

	if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) {
		innopwr_notice("run finished and init vol is %d is_digital = %d is_support_update_voltage = %d\n",
			init_vol, hal_vol_is_digital(power->pdev), is_support_update_voltage());
	}

	return 0;

power_fail:
	devm_kfree(dev, power);
	innopwr_err("power_fail\n");

	return -ENOMEM;
}

static int innopower_pmbus_remove(struct platform_device *pdev)
{
	struct innopower *power = platform_get_drvdata(pdev);
	struct device *dev = &pdev->dev;

	innopower_sys_unregister(&pdev->dev);

	innopower_vol_init(power->pdev, PWR_NOTIFY_FOR_NORMAL);

	innopower_unregister_chip(power);

	devm_kfree(dev, power);

	if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) {
		innopwr_notice("run finished\n");
	}

	return 0;
}

static void innopower_pmbus_shutdown(struct platform_device *pdev)
{
	struct innopower *power = platform_get_drvdata(pdev);
	innopower_vol_init(power->pdev, PWR_NOTIFY_FOR_SHUTDOWN);
	if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) {
		innopwr_notice("run finished\n");
	}
}
struct platform_device_id innopower_pmbus_device_id_table[] = {
	{ .name = INNO_POWER_DEVICE_NAME, .driver_data = 0 },
	{},
};

static int innopwr_enter_sleep(struct device *dev, char *msg)
{
	int ret = 0;
	struct innopower *power = platform_get_drvdata(to_platform_device(dev));
	struct dev_rsrc *pdev_rsrc = inno_rsrc_devres_find(power->pdev);
	uint32_t sleep_gpu_freq = hal_get_pll(pdev_rsrc->dev, PLL_GPU);
	uint32_t real_vol = 0;

	if (!is_support_update_voltage()) {
		return 0;
	}

	if (hal_vol_is_digital(pdev_rsrc->dev)) {
		if (innopwr_drv_mcufw_get_voltage_from_reg(pdev_rsrc, &real_vol)) {
			innopwr_notice("%s read voltage failed and sleep_vol[%d]\n", msg, pdev_rsrc->sleep_vol);
			return -1;
		}

	} else {
		real_vol = ANALOG_VOLT_DEFAULT_OUTPUT;
	}
	pdev_rsrc->sleep_vol = innopwr_find_vol(pdev_rsrc->dev, sleep_gpu_freq);

	pdev_rsrc->sleep_freq = sleep_gpu_freq;
	if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) {
		innopwr_notice("%s run finished and opp[%u, %d %u]\n", msg, sleep_gpu_freq, pdev_rsrc->sleep_vol, real_vol);
	}

	return ret;
}

static int innopwr_exit_sleep(struct device *dev, char *msg, uint8_t scene)
{
	int ret = 0;
	uint32_t real_vol = 0;
	struct innopower *power = platform_get_drvdata(to_platform_device(dev));
	struct dev_rsrc *pdev_rsrc = inno_rsrc_devres_find(power->pdev);
	uint32_t gpu_freq = hal_get_pll(pdev_rsrc->dev, PLL_GPU);

	if (!is_support_update_voltage()) {
		return 0;
	}

	if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) {
		if (hal_vol_is_digital(pdev_rsrc->dev)) {
			if (!innopwr_drv_mcufw_get_voltage_from_reg(pdev_rsrc, &real_vol)) {
				innopwr_notice("%s bf run start and opp[%u, %u | %u, %d]\n",
						msg, gpu_freq, real_vol, pdev_rsrc->sleep_freq, pdev_rsrc->sleep_vol);
			} else {
				innopwr_notice("%s bf read voltage failed\n", msg);
			}
		} else {
			real_vol = ANALOG_VOLT_DEFAULT_OUTPUT;
		}
	}

	if (pdev_rsrc->sleep_vol > 0) {
		if (gpu_freq < pdev_rsrc->sleep_freq) { //increase freq
			innopower_set_voltage(pdev_rsrc, pdev_rsrc->sleep_vol, scene);
		}

		hal_set_pll(pdev_rsrc->dev, PLL_GPU, pdev_rsrc->sleep_freq);

		if (gpu_freq >= pdev_rsrc->sleep_freq) { //decrease freq
			innopower_set_voltage(pdev_rsrc, pdev_rsrc->sleep_vol, scene);
		}
	}

	if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) {
		gpu_freq = hal_get_pll(pdev_rsrc->dev, PLL_GPU);
		if (hal_vol_is_digital(pdev_rsrc->dev)) {
			if (!innopwr_drv_mcufw_get_voltage_from_reg(pdev_rsrc, &real_vol)) {
				innopwr_notice("%s af run finished and opp[%u, %u | %u, %d]\n",
						msg, gpu_freq, real_vol, pdev_rsrc->sleep_freq, pdev_rsrc->sleep_vol);
			} else {
				innopwr_notice("%s af read voltage failed\n", msg);
			}
		} else {
			innopwr_notice("%s bf analog power real_vol is %d\n", msg, real_vol);
		}
	}

	return ret;
}

static int innopwr_suspend(struct device *dev)
{
	innopwr_enter_sleep(dev, "s3 suspend");
	return 0;
}

static int innopwr_resume(struct device *dev)
{
	innopwr_exit_sleep(dev, "s3 resume", PWR_NOTIFY_FOR_S3_EXIT);
	return 0;
}

static int innopwr_freeze(struct device *dev)
{
	innopwr_enter_sleep(dev, "s4 freeze");
	return 0;
}

static int innopwr_thaw(struct device *dev)
{
	int ret = 0;

	if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) {
		innopwr_notice("run finished\n");
	}

	return ret;
}
static int innopwr_poweroff(struct device *dev)
{
	int ret = 0;
	struct innopower *power = platform_get_drvdata(to_platform_device(dev));

	innopower_vol_init(power->pdev, PWR_NOTIFY_FOR_POWEROFF);
	if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) {
		innopwr_notice("run finished\n");
	}

	return ret;
}

static int innopwr_restore(struct device *dev)
{
	innopwr_exit_sleep(dev, "s4 restore", PWR_NOTIFY_FOR_S4_EXIT);
	return 0;
}

const static struct dev_pm_ops innopwr_pm_ops = {
	.suspend = innopwr_suspend,
	.resume = innopwr_resume,
	.freeze = innopwr_freeze,
	.thaw = innopwr_thaw,
	.poweroff = innopwr_poweroff,
	.restore = innopwr_restore,
};


struct platform_driver innopower_pmbus_driver = {
	.probe = innopower_pmbus_probe,
	.remove = innopower_pmbus_remove,
	.shutdown = innopower_pmbus_shutdown,
	.driver = {
		.name = INNO_POWER_DEVICE_NAME,
		.pm = &innopwr_pm_ops,
	},
	.id_table = innopower_pmbus_device_id_table,
};

#ifdef CONFIG_DRM_INNO_POWER
int innopower_driver_register(void)
#else
static int __init innopower_init(void)
#endif
{
	platform_driver_register(&innopower_pmbus_driver);
	if (i2c_drv) {
		i2c_drv->id_table = inno_powerchip_id_tbl;
		i2c_add_driver(i2c_drv);
	}

#ifdef CONFIG_PM_DEVFREQ
	if (inno_devfreq_gov_register())
		innopwr_err("failed register governor\n");
#endif

	return 0;
}

#ifdef CONFIG_DRM_INNO_POWER
void innopower_driver_unregister(void)
#else
static void __exit innopower_exit(void)
#endif
{
#ifdef CONFIG_PM_DEVFREQ
	if (inno_devfreq_gov_unregister())
		innopwr_err("failed remove governor\n");
#endif
	if (i2c_drv) {
		i2c_del_driver(i2c_drv);
	}

	platform_driver_unregister(&innopower_pmbus_driver);
}

#ifndef CONFIG_DRM_INNO_POWER
module_init(innopower_init);
module_exit(innopower_exit);
MODULE_LICENSE("Dual MIT/GPL");
#endif
