#ifndef INNO_POWER_H
#define INNO_POWER_H

#include "hal.h"
#include <linux/version.h>

#define INNO_POWER_ERR -1
#define INNO_POWER_OK	0

#define POWER_CHIP_NAME	20

#define INNOPWR "innopwr"
#define pr_power(fmt) "[%s][%s:%d]" fmt,INNOPWR,__func__,__LINE__
#if defined(INNO_GPU_LOG)
#define innopwr_dbg(fmt, ...)\
		printk(KERN_DEBUG pr_power(fmt), ##__VA_ARGS__)
#define innopwr_info(fmt, ...)\
		printk(KERN_INFO pr_power(fmt), ##__VA_ARGS__)
#else
#define innopwr_dbg(fmt, ...)
#define innopwr_info(fmt, ...)
#endif

#define innopwr_notice( fmt, ...) \
		printk(KERN_NOTICE pr_power(fmt), ##__VA_ARGS__)
#define innopwr_warn( fmt, ...) \
		printk(KERN_WARNING pr_power(fmt), ##__VA_ARGS__)
#define innopwr_err(fmt, ...)\
		printk(KERN_NOTICE pr_power(fmt), ##__VA_ARGS__)

struct powerchip {
	char name[POWER_CHIP_NAME];

	struct i2c_client *client;
	int (*get_gpu_voltage)(struct powerchip *pwrchip, int channel);
	int (*set_gpu_voltage)(struct powerchip *pwrchip, int channel, unsigned vol);

	struct mutex lock;
	void *priv_data;
};

struct innopower {
	struct volctrl_chip_hwinfo_t *params;
	struct i2c_adapter *adapter;

	struct device *pdev;	//pci_dev->dev
	struct device *pltdev;

	struct powerchip *chip;
};

int innopwr_read8(struct powerchip *pwrchip, unsigned char reg);
int innopwr_write8(struct powerchip *pwrchip, unsigned char reg, unsigned char data);
int innopwr_read16(struct powerchip *pwrchip, unsigned char reg);
int innopwr_write16(struct powerchip *pwrchip, unsigned char reg, unsigned int data);

int innopwr_read8_no_repeat_start(struct powerchip *pwrchip, unsigned char reg);
int innopwr_write8_no_repeat_start(struct powerchip *pwrchip, unsigned char reg, unsigned char data);
int innopwr_read16_no_repeat_start(struct powerchip *pwrchip, unsigned char reg);
int innopwr_write16_no_repeat_start(struct powerchip *pwrchip, unsigned char reg, unsigned int data);

int innopower_driver_register(void);
void innopower_driver_unregister(void);
#include "power_hw_info.h"
#endif	/*INNO_POWER_H*/
