/*************************************************************************/ /*!
@File			innodpu_print.h
@Title
@Copyright		Copyright (c) Innosilicon Technology Ltd. All Rights Reserved
@Description
@License		Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef __INNODPU_PRINTK_H
#define __INNODPU_PRINTK_H
#include <linux/compiler.h>
#include <linux/seq_file.h>
#include <linux/device.h>
#include <linux/seq_file.h>
#include "inno_drm_version.h"
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0))
#include <drm/drm_print.h>
#endif

#include "inno_plat_dev.h"
#include "inno_fs.h"
#include "inno_debug.h"

#define INNODPU_WARN_RETURN_CHECK(_cond, _ret) do { \
	if (!(_cond)) { \
		inno_printk(KERN_DEFAULT "[inno:%ps] trigger assert at line %d\n", __builtin_return_address(0), __LINE__); \
		inno_warn_on(0); \
		return (_ret); \
	} \
} while (0)

#define DPU_UT_NONE		0x00
#define DPU_UT_INIT		0x01
#define DPU_UT_CORE		0x02
#define DPU_UT_MEM		0x04
#define DPU_UT_DPU		0x08
#define DPU_UT_HDMI		0x10
#define DPU_UT_DP		0x20
#define DPU_UT_LVDS		0x40
#define DPU_UT_VGA		0x80
#define DPU_UT_CURSOR	0x100
#define DPU_UT_GAMMA	0x200


extern unsigned int s_dpu_debug;
extern unsigned int s_dpu_level;
extern void innodpu_debug(inno_dev *dev, unsigned int category, const char *format, ...);
extern void innodpu_info(inno_dev *dev, unsigned int category, const char *format, ...);
extern void innodpu_warn(inno_dev *dev, const char *format, ...);
extern void innodpu_err(inno_dev *dev, const char *format, ...);
extern void inno_seq_printf(inno_seq_file *m, const char *f, ...);



#endif //__INNODPU_PRINTK_H
