#include "innodpu_common.h"
#include "hal_interface.h"
#include "innodpu_compatibility.h"
#include "innodpu_print.h"

#if (DRM_VERSION >= KERNEL_VERSION(5, 19, 0))
#include <drm/display/drm_dp_helper.h>
#else
#include <drm/drm_dp_helper.h>
#endif

#include "innodpu_dp.h"
#include "innodpu_dp_common.h"
#include "innodpu_connector.h"

#define INNO_G0_DP_NUM  3
#define INNO_G0M_DP_NUM  3
#define INNO_G1_DP_NUM  3
#define INNO_G1P_DP_NUM 3

struct dp_ext_t {
	inno_waitqueue_head *dp_enc_wq;
	atomic64_t enc_status;
	inno_waitqueue_head *dp_dec_wq;
	atomic64_t dec_status;

	inno_waitqueue_head *dp_aux_wq;
	atomic64_t aux_status;

	spinlock_t intr_slock;

	atomic64_t hpg_status;

	atomic64_t audio_status;

	struct i2c_adapter adapter;
	struct i2c_algo_dp_aux_data algo;

	struct edid *edid;
};

struct dp_ext_t g0_dp_ext[INNO_G0_DP_NUM];
struct dp_ext_t g0m_dp_ext[INNO_G0M_DP_NUM];
struct dp_ext_t g1_dp_ext[INNO_G1_DP_NUM];
struct dp_ext_t g1p_dp_ext[INNO_G1P_DP_NUM];

void dp_set_aux_status(struct dp_chip_t *chip, int val)
{
	atomic64_set(&chip->dp_ext->aux_status, val);
}

long long dp_get_aux_status(struct dp_chip_t *chip)
{
	return atomic64_read(&chip->dp_ext->aux_status);
}

void dp_set_hpg_status(struct dp_chip_t *chip, int val)
{
	atomic64_set(&chip->dp_ext->hpg_status, val);
}
long long dp_get_hpg_status(struct dp_chip_t *chip)
{
	return atomic64_read(&chip->dp_ext->hpg_status);
}

void dp_set_audio_status(struct dp_chip_t *chip, int val)
{
	atomic64_set(&chip->dp_ext->audio_status, val);
}
long long dp_get_audio_status(struct dp_chip_t *chip)
{
	return atomic64_read(&chip->dp_ext->audio_status);
}

void *dp_get_edid(struct dp_chip_t *chip)
{
	return chip->dp_ext->edid;
}
void dp_set_edid(struct dp_chip_t *chip, void *val)
{
	chip->dp_ext->edid = (struct edid *)val;
}

int dp_ext_init(struct dp_chip_t *chip)
{
	chip_type_e plat;

	plat = hal_get_chiptype(inno_dev_get_parent(chip->dev));

	switch(plat) {
	case CHIP_G0_SOC:
		if(chip->id > INNO_G0_DP_NUM)
			return -1;
		memset(&g0_dp_ext[chip->id], 0, sizeof(struct dp_ext_t));
		chip->dp_ext = &g0_dp_ext[chip->id];
		break;
	case CHIP_G0M_SOC:
		if(chip->id > INNO_G0M_DP_NUM)
			return -1;
		memset(&g0m_dp_ext[chip->id], 0, sizeof(struct dp_ext_t));
		chip->dp_ext = &g0m_dp_ext[chip->id];
		break;
	case CHIP_G1P_SOC:
		if(chip->id > INNO_G1P_DP_NUM)
			return -1;
		memset(&g1p_dp_ext[chip->id], 0, sizeof(struct dp_ext_t));
		chip->dp_ext = &g1p_dp_ext[chip->id];
		break;
	case CHIP_G1_SOC:
		if(chip->id > INNO_G1_DP_NUM)
			return -1;
		memset(&g1_dp_ext[chip->id], 0, sizeof(struct dp_ext_t));
		chip->dp_ext = &g1_dp_ext[chip->id];
		break;
	default:
		innodpu_err(chip->dev, "dp ext init error");
		return -1;
		break;
	}
	return 0;
}

