/*************************************************************************/ /*!
@File			innovpu.h
@Title			innovpu driver header
@Copyright		Copyright (c) Innosilicon Technology Ltd. All Rights Reserved
@Description	innovpu driver header
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef __VPU_DRV_H__
#define __VPU_DRV_H__

#include <linux/fs.h>
#include <linux/types.h>
#include <linux/kfifo.h>
//#include <linux/cdev.h>
#include <linux/miscdevice.h>
#include <linux/platform_device.h>
#include <linux/cdev.h>
#include <linux/suspend.h>
#include "hal.h"
#include "hal_interface.h"
#include "inno_waitqueue.h"
#include "inno_debug.h"
#include "inno_idr.h"

#define INNO_VPU_DEBUGFS_NAME "innovpu"

#if defined(linux) || defined(__linux) || defined(ANDROID)
#define SUPPORT_MULTI_INST_INTR
#endif

/* if the driver want to use interrupt service from kernel ISR */
#ifdef SUPPORT_INTERRUPT
#define VPU_SUPPORT_ISR
#else
#endif

#ifdef VPU_SUPPORT_ISR
/* if the driver want to disable and enable IRQ whenever interrupt asserted. */
//#define VPU_IRQ_CONTROL
#endif

#ifdef VPU_SUPPORT_ISR
#define VPU_IRQ_NUM (23+32)
#endif

#define INNO_CMD_REG_END                0x00000200
#define VPU_RST_BLOCK_ALL               (0x3fffffff)
#define VPU_MAX_LOAD_SIZE (3840 * 2160 * 60)

#define MAX_INST_HANDLE_SIZE            56              /* DO NOT CHANGE THIS VALUE */
#define MAX_NUM_INSTANCE                32
#define MAX_NUM_VPU_CORE                1
#define MAX_NUM_VCORE                   1
#define ATOMIC_SYNC_TIMEOUT             (60000)
#define BIT_CODE_LEN                    512
#define VPU_CMD_REG_LEN                 512

#define VPU_FOUR_BYTES_ANDOPERATION     (0xffffffff)

#ifndef VM_RESERVED	/*for kernel up to 3.7.0 version*/
# define  VM_RESERVED   (VM_DONTEXPAND | VM_DONTDUMP)
#endif

#ifdef VPU_SUPPORT_RESERVED_VIDEO_MEMORY
#	define VPU_INIT_VIDEO_MEMORY_SIZE_IN_BYTE (62*1024*1024)
#	define VPU_DRAM_PHYSICAL_BASE 0x86C00000
#include "vpu/vmm.h"
#endif /*VPU_SUPPORT_RESERVED_VIDEO_MEMORY*/

#define VPU_SUPPORT_PLATFORM_DRIVER_REGISTER

#define KBUILD_VPU "inno_vpu"
#define pr_fmt_vpu(fmt) "[%s][%s:%d]" fmt,KBUILD_VPU,__func__,__LINE__

//#define VPU_DEBUG
#ifndef VPU_DEBUG
#define vpu_dbg(dev,fmt, ...)
#define vpu_info(dev,fmt, ...)
#else
#define vpu_dbg(dev,fmt, ...)\
		inno_dev_printk(KERN_DEBUG,dev,pr_fmt_vpu(fmt), ##__VA_ARGS__)
#define vpu_info(dev,fmt, ...)\
		inno_dev_printk(KERN_INFO,dev,pr_fmt_vpu(fmt), ##__VA_ARGS__)
#endif
#define vpu_warn(dev,fmt, ...)\
		inno_dev_printk(KERN_WARNING,dev,pr_fmt_vpu(fmt), ##__VA_ARGS__)
#define vpu_error(dev,fmt, ...)\
		inno_dev_printk(KERN_ERR,dev,pr_fmt_vpu(fmt), ##__VA_ARGS__)

#define vpu_prinfo(fmt, ...) inno_info(fmt, ##__VA_ARGS__)
#define vpu_prerr(fmt, ...) inno_error(fmt, ##__VA_ARGS__)


#define VPU_WAKE_MODE           0
#define VPU_SLEEP_MODE          1
#define VPU_NAME_SIZE           10
#define VDI_NUM_LOCK_HANDLES    5
#define REG_SUCCESS             (0)


#define VDI_IOCTL_MAGIC  'V'
#define VDI_IOCTL_ALLOCATE_PHYSICAL_MEMORY	_IO(VDI_IOCTL_MAGIC, 0)
#define VDI_IOCTL_FREE_PHYSICALMEMORY		_IO(VDI_IOCTL_MAGIC, 1)
#define VDI_IOCTL_WAIT_INTERRUPT			_IO(VDI_IOCTL_MAGIC, 2)
#define VDI_IOCTL_SET_CLOCK_GATE			_IO(VDI_IOCTL_MAGIC, 3)
#define VDI_IOCTL_RESET						_IO(VDI_IOCTL_MAGIC, 4)
#define VDI_IOCTL_GET_INSTANCE_POOL			_IO(VDI_IOCTL_MAGIC, 5)
#define VDI_IOCTL_GET_COMMON_MEMORY			_IO(VDI_IOCTL_MAGIC, 6)
#define VDI_IOCTL_GET_RESERVED_VIDEO_MEMORY_INFO _IO(VDI_IOCTL_MAGIC, 8)
#define VDI_IOCTL_OPEN_INSTANCE				_IO(VDI_IOCTL_MAGIC, 9)
#define VDI_IOCTL_CLOSE_INSTANCE			_IO(VDI_IOCTL_MAGIC, 10)
#define VDI_IOCTL_GET_INSTANCE_NUM			_IO(VDI_IOCTL_MAGIC, 11)
#define VDI_IOCTL_GET_REGISTER_INFO			_IO(VDI_IOCTL_MAGIC, 12)
#define VDI_IOCTL_GET_FREE_MEM_SIZE			_IO(VDI_IOCTL_MAGIC, 13)
#define VDI_IOCTL_DMABUF_EXPORT				_IO(VDI_IOCTL_MAGIC, 14)
#define VDI_IOCTL_DMABUF_IMPORT				_IO(VDI_IOCTL_MAGIC, 15)
#define VDI_IOCTL_DMABUF_DESTROY			_IO(VDI_IOCTL_MAGIC, 16)
#define VDI_IOCTL_GET_PVRIC_MEMORY			_IO(VDI_IOCTL_MAGIC, 17)/**TODO: why*/
#define VDI_IOCTL_DMAFENCE_SIGNAL			_IO(VDI_IOCTL_MAGIC, 18)
#define VDI_IOCTL_DMAFENCE_CREATE			_IO(VDI_IOCTL_MAGIC, 19)
#define VDI_IOCTL_DMA_TRANSFER				_IO(VDI_IOCTL_MAGIC, 20)
#define VDI_IOCTL_GET_CHIP_INFO				_IO(VDI_IOCTL_MAGIC, 21)
#define VDI_IOCTL_LOCK_ENQUEUE				_IO(VDI_IOCTL_MAGIC, 22)
#define VDI_IOCTL_LOCK_DEQUEUE				_IO(VDI_IOCTL_MAGIC, 23)
#define VDI_IOCTL_TRY_LOCK					_IO(VDI_IOCTL_MAGIC, 24)
#define VDI_IOCTL_UNLOCK					_IO(VDI_IOCTL_MAGIC, 25)
#define VDI_IOCTL_CREATE_INSTANCE			_IO(VDI_IOCTL_MAGIC, 26)
#define VDI_IOCTL_DESTROY_INSTANCE			_IO(VDI_IOCTL_MAGIC, 27)
#define VDI_IOCTL_INCREASE_LOAD				_IO(VDI_IOCTL_MAGIC, 28)
#define VDI_IOCTL_GET_VRAM_INFO				_IO(VDI_IOCTL_MAGIC, 29)
#define VDI_IOCTL_DMA_MAP_HOSTMEM			_IO(VDI_IOCTL_MAGIC, 30)
#define VDI_IOCTL_DMA_UNMAP_HOSTMEM			_IO(VDI_IOCTL_MAGIC, 31)
#define VDI_IOCTL_DMA_TRANSFER_FAST			_IO(VDI_IOCTL_MAGIC, 32)

#define INNO_VPU_GEM_DOMAIN_CPU       (1 << 0)
#define INNO_VPU_GEM_DOMAIN_VRAM      (1 << 1)

#define INNO_GPU_BAR_SIZE_SMALL       (1 << 0)
#define INNO_GPU_BAR_SIZE_2G	      (1 << 1)
#define INNO_GPU_BAR_SIZE_4G	      (1 << 2)

#define PADDING_LOW   4

#define CLEAR_BIT(val, bit) (val &= ~(1<<bit))
#define SET_BIT(val, bit) (val |= (1<<bit))

#define SLC_IDX0 0
#define SLC_IDX1 1
#define SLC_IDX2 2
#define SLC_IDX3 3
#define USE_SLC_EN  1

#define VPU_SYSERR_VPU_STILL_RUNNING        (0x00001000)
#define ERR_RESULT_NOT_READY                (0x00000800)
#define RUN_CMD_STATUS_FAIL                 (0x0)
#define VPU_RESET_STATUS_RELEASED           (0x0)
#define VPU_BUSY_STATUS_IDLE                (0x0)

/*
IMPORTANT:
All structures below are designed to be the same size when compiled for 32
and/or 64 bit architectures, i.e. there should be no compiler inserted
padding. This is achieved by sticking to the following rules:
1) only use fixed width types
2) always naturally align fields by arranging them appropriately and by using
padding fields when necessary
*/
typedef struct vpudrv_buffer_t {
	u64 phys_addr;
	u64 base;					/* kernel logical address in use kernel */
	u64 virt_addr;				/* virtual user space address */
	u64 dev_addr;
	int size;
	int fd;
	u32 domain;
	int id;
	// char reserved[PADDING_LOW];
} vpudrv_buffer_t;

typedef struct vpu_bit_firmware_info_t {
	unsigned int size;						/* size of this structure*/
	unsigned int core_idx;
	u64 reg_base_offset;
	unsigned short bit_code[BIT_CODE_LEN];
} vpu_bit_firmware_info_t;

typedef struct vpudrv_inst_info_t {
	int core_idx;
	int inst_idx;
	int inst_open_count;	/* for output only*/
	char reserved[PADDING_LOW];
} vpudrv_inst_info_t;

typedef struct vpudrv_intr_info_t {
	int timeout;
	int intr_reason;
#ifdef SUPPORT_MULTI_INST_INTR
	int intr_inst_index;
	char reserved[PADDING_LOW];
#endif
} vpudrv_intr_info_t;

typedef struct vpu_context_t {
	struct fasync_struct *async_queue;
#ifdef SUPPORT_MULTI_INST_INTR
	u64 interrupt_reason[MAX_NUM_INSTANCE];
#else
	u64 interrupt_reason;
#endif
	u32 open_count;					 /*!<< device reference count. Not instance count */
} vpu_context_t;

typedef enum {
	VPU_ID_0,
	VPU_ID_1,
	VPU_ID_2,
	VPU_ID_3,
	VPU_ID_MAX,
} vpu_id;

typedef enum {
	VPU_INITIALIZE,
	VPU_NOT_INITIALIZE,
}VPU_STATUS;

typedef struct kfifo kfifo_t;

typedef struct vpudrv_chip_info_t {
	u64 host_bar2_base_addr;
	u64 host_bar2_end_addr;
	u64 vram_dev_base_addr;
	u64 vram_dev_end_addr;
	u32 chip_type;
	u32 vpu_num;
	u32 bar_size;
	u32 vf_id;
	char reserved[PADDING_LOW];
} vpudrv_chip_info_t;

typedef struct vpu_drv_ctx {
	vpudrv_buffer_t instance_pool;
	vpudrv_buffer_t common_memory;
	vpudrv_buffer_t pvric_memory;
	vpudrv_buffer_t vpu_register;
	vpu_context_t   vpu_context;
	vpudrv_chip_info_t chip_info;
	vpu_bit_firmware_info_t bit_firmware_info[MAX_NUM_VPU_CORE];
	u32 interrupt_flag[MAX_NUM_INSTANCE];
	u32 vpu_reg_store[MAX_NUM_VPU_CORE][VPU_CMD_REG_LEN];
	u32 inst_lock_wake_up_flag[MAX_NUM_INSTANCE];
	kfifo_t           *interrupt_pending_q[MAX_NUM_INSTANCE];
	wait_queue_head_t *interrupt_wait_q[MAX_NUM_INSTANCE];
	wait_queue_head_t *inst_lock_wait_q[MAX_NUM_INSTANCE];
	spinlock_t        *inst_lock_info_lock[MAX_NUM_INSTANCE];
	u32 vpu_id;
	u32 product_code;
	int irq_handler_num;
	u32 vpu_open_ref_count;
	bool is_sharing_gpu_heap;
	inno_dev *dev;
	inno_dev *parent;
	inno_dev *vpudev;
	dev_t *cdevid;
	struct class *vpucls;
	struct dentry *vpu_dir;
	struct dentry *vpu_node;
	struct semaphore *vpu_sem;
	spinlock_t *kfifo_lock;
	spinlock_t *vpu_lock;
	spinlock_t *inst_index_lock;  //control the selection of instance ID
	struct list_head inst_lock_info_head[MAX_NUM_INSTANCE];
	struct list_head vbp_head;
	struct list_head pm_buf_head;
	struct list_head inst_list_head;
	char vpuname[VPU_NAME_SIZE];
	atomic64_t invisible_mem;
	atomic64_t visible_mem;
	atomic64_t external_mem;
	atomic64_t statistic_load;
	struct cdev vpucdev;
	inno_idr* hostmem_buffers;
#ifdef CONFIG_PM_SLEEP
	struct notifier_block pm_notifier;
#endif
}vpu_drv_context;

typedef struct vpu_buf_obj {
	vpudrv_buffer_t vb;
	vpu_drv_context *ctx;
	struct dma_buf_attachment *import_attach;
	struct sg_table *sgt;
	struct dma_buf *buf;
}vpu_buf_obj_t;

/* To track the allocated memory buffer */
typedef struct vpudrv_buffer_pool_t {
	struct list_head list;
	union {
		vpudrv_buffer_t vb;
		vpu_buf_obj_t obj;
	} buf;
	void *filp;
} vpudrv_buffer_pool_t;

/* To track the pm memory buffer */
typedef struct pm_buf_pool_t {
	struct list_head list;
	void* virt_addr;
	u64 phys_addr;
	u32 size;
	u32 domain;
} pm_buf_pool_t;

typedef struct{
	u32 width;
	u32 height;
	u32 frameRate;
	char reserved[PADDING_LOW];
} vpu_instance_param_t;

/* To track the instance index and buffer in instance pool */
typedef struct vpudrv_instanace_list_t {
	struct list_head list;
	int inst_idx;
	int core_idx;
	void *filp;
	vpu_instance_param_t inst_param;
} vpudrv_instanace_list_t;

typedef struct vpudrv_instance_pool_t {
	unsigned char   codecInstPool[MAX_NUM_INSTANCE][MAX_INST_HANDLE_SIZE];
	vpudrv_buffer_t vpu_common_buffer;
	u32 vpu_instance_num;
	u32 instance_pool_inited;
	void* pendingInst;
	u32 pendingInstIdxPlus1;
	u32 doSwResetInstIdxPlus1;
} vpudrv_instance_pool_t;

typedef enum {
	VPUAPI_RET_SUCCESS,
	VPUAPI_RET_FAILURE, // an error reported by FW
	VPUAPI_RET_TIMEOUT,
	VPUAPI_RET_STILL_RUNNING,
	VPUAPI_RET_INVALID_PARAM,
	VPUAPI_RET_NOT_READY,
	VPUAPI_RET_MAX
} vpu_api_ret;

typedef enum {
	VPUDRV_MUTEX_VPU,
	VPUDRV_MUTEX_DISP_FALG,
	VPUDRV_MUTEX_RESET,
	VPUDRV_MUTEX_VMEM,
	VPUDRV_MUTEX_REV1,
	VPUDRV_MUTEX_MAX
} vpudrv_mutex_type;

enum {
	DEC_SEQ_INIT = 1,
	ENC_SEQ_INIT = 1,
	DEC_SEQ_END  = 2,
	ENC_SEQ_END  = 2,
	PIC_RUN      = 3,
	SET_FRAME_BUF = 4,
	ENCODE_HEADER = 5,
	ENC_PARA_SET  = 6,
	DEC_PARA_SET  = 7,
	DEC_BUF_FLUSH = 8,
	RC_CHANGE_PARAMETER = 9,
	VPU_SLEEP           = 10,
	VPU_WAKE            = 11,
	ENC_ROI_INIT        = 12,
	FIRMWARE_GET        = 0xf
};

typedef struct{
	vpudrv_buffer_t src_vb;
	vpudrv_buffer_t dest_vb;
	int size;
	int direction;
} vpu_dma_info_t;

typedef struct{
	u64 instIndex;
	u64 timestamp;
} vpu_lock_info_t;

typedef struct{
	struct list_head list;
	u64 instIndex;
	u64 timestamp;
} vpu_lock_info_list_t;

typedef enum {
	INT_VPU_INIT_VPU          = 0,
	INT_VPU_WAKEUP_VPU        = 1,
	INT_VPU_SLEEP_VPU         = 2,
	INT_VPU_CREATE_INSTANCE   = 3,
	INT_VPU_FLUSH_INSTANCE    = 4,
	INT_VPU_DESTROY_INSTANCE  = 5,
	INT_VPU_INIT_SEQ          = 6,
	INT_VPU_SET_FRAMEBUF      = 7,
	INT_VPU_DEC_PIC           = 8,
	INT_VPU_ENC_PIC           = 8,
	INT_VPU_ENC_SET_PARAM     = 9,
	INT_VPU_DEC_QUERY         = 14,
	INT_VPU_BSBUF_EMPTY       = 15,
	INT_VPU_BSBUF_FULL        = 15,
} VPU_interrupt_bit;

typedef enum {
    VPU_INIT_VPU        = 0x0001,
    VPU_WAKEUP_VPU      = 0x0002,
    VPU_SLEEP_VPU       = 0x0004,
    VPU_CREATE_INSTANCE = 0x0008,            /* queuing command */
    VPU_FLUSH_INSTANCE  = 0x0010,
    VPU_DESTROY_INSTANCE= 0x0020,            /* queuing command */
    VPU_INIT_SEQ        = 0x0040,            /* queuing command */
    VPU_SET_FB          = 0x0080,
    VPU_DEC_PIC         = 0x0100,            /* queuing command */
    VPU_ENC_PIC         = 0x0100,            /* queuing command */
    VPU_ENC_SET_PARAM   = 0x0200,            /* queuing command */
    VPU_QUERY           = 0x4000,
    VPU_UPDATE_BS       = 0x8000,
    VPU_RESET_VPU	   = 0x10000,
    VPU_MAX_VPU_COMD	= 0x10000,
} VPU_vpu_command;

typedef enum {
    GET_VPU_INFO        = 0,
    SET_WRITE_PROT      = 1,
    GET_RESULT          = 2,
    UPDATE_DISP_FLAG    = 3,
    GET_BW_REPORT       = 4,
    GET_BS_RD_PTR       = 5,    // for decoder
    GET_BS_WR_PTR       = 6,    // for encoder
    GET_SRC_BUF_FLAG    = 7,    // for encoder
    SET_BS_RD_PTR       = 8,    // for decoder
    GET_DEBUG_INFO      = 0x61,
} query_opt;

typedef struct sync_event{
	int fd;
	bool event;/*write：1， read：0*/
} sync_event_t;

#endif

