/*************************************************************************/ /*!
@File			innovpu_common.h
@Title			innovpu common driver header
@Copyright		Copyright (c) Innosilicon Technology Ltd. All Rights Reserved
@Description	innovpu common driver header
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#include "hal_interface.h"
#include "innovpu.h"

#define HOSTMEM_MIN_BUFFER 1
#define HOSTMEM_MAX_BUFFER 4096

u32 vpu_get_product_code(vpu_drv_context *vpu_drv_ctx);
#ifdef USE_CODEC_NAME//reserved todo...
void vpu_set_vpudev_name(inno_dev *vpudev, u32 product_code, char *name);
void vpu_reset_vpudev_num(inno_dev *vpudev, u32 product_code);
#endif
u64 vpu_get_free_mem_size(vpu_drv_context *vpu_drv_ctx);
void vpu_common_memory_addr_set(vpu_drv_context *vpu_drv_ctx);
int vpu_dma_transfer(vpu_drv_context *vpu_drv_ctx, vpu_dma_info_t info);
int vpu_alloc_dma_buffer(vpudrv_buffer_t *vb, vpu_drv_context *vpu_drv_ctx);
void vpu_free_dma_buffer(vpudrv_buffer_t *vb, vpu_drv_context *vpu_drv_ctx);
u32 vpu_free_lock(vpu_drv_context *vpu_drv_ctx, void *filp);
u32 vpu_free_instances(vpu_drv_context *vpu_drv_ctx, void *filp);
u32 vpu_sleep_wake(u32 core, u32 mode, vpu_drv_context *vpu_drv_ctx);
void vpu_handle_irq(vpu_drv_context *vpu_drv_ctx);
int vpu_suspend(vpu_drv_context *vpu_drv_ctx);
int vpu_resume(vpu_drv_context *vpu_drv_ctx);
int vpu_hw_reset(vpu_drv_context *vpu_drv_ctx);
int vpu_sw_reset(vpu_drv_context *vpu_drv_ctx);
int vpu_thaw(vpu_drv_context *vpu_drv_ctx);
int vpu_poweroff(vpu_drv_context *vpu_drv_ctx);
int vpu_lock_enqueue(vpu_drv_context *vpu_drv_ctx, vpu_lock_info_t *lock_info);
int vpu_lock_dequeue(vpu_drv_context *vpu_drv_ctx, u64 instIndex);
int vpu_try_lock(vpu_drv_context *vpu_drv_ctx, u64 instIndex);
int vpu_unlock(vpu_drv_context *vpu_drv_ctx, u64 instIndex);
int vpu_create_instance(vpu_drv_context *vpu_drv_ctx, u64 instIndex);
int vpu_destroy_instance(vpu_drv_context *vpu_drv_ctx, u64 instIndex);
int vpu_capture_inst_index(vpu_drv_context *vpu_drv_ctx, u32 core);//get instIndex from codecInstPool
int vpu_release_inst_index(vpu_drv_context *vpu_drv_ctx, u32 core, u64 instIndex);
int vpu_mutex_lock(u32 core, u32 inst, u32 type, vpu_drv_context *vpu_drv_ctx);
void vpu_mutex_unlock(u32 core, u32 inst, u32 type, vpu_drv_context *vpu_drv_ctx);
int vpu_hw_res_init(vpu_drv_context *vpu_drv_ctx);
u32 vpu_get_product_code_manual(vpu_drv_context * vpu_drv_ctx);
int vpu_dma_map_hostmem(vpu_drv_context *vpu_drv_ctx, vpudrv_buffer_t *vb);
void vpu_dma_unmap_hostmem(vpu_drv_context *vpu_drv_ctx, vpudrv_buffer_t *vb);
int vpu_idr_dma_unmap_hostmem(int id, void *ptr, void *data);
int vpu_dma_transfer_fast(vpu_drv_context *vpu_drv_ctx, vpu_dma_info_t info);
int vpu_notifier(struct notifier_block *nb, unsigned long action, void *data);
