#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <string.h>

#define GTEST_DMA_DEV "/dev/gtest_dma_test_dev"
#define GTEST_DMA_CDEV_SIZE	512

#define DMA_IOC_MAGIC	't'
#define DMA_DDR2SYS		_IO(DMA_IOC_MAGIC, 1)
#define DMA_SYS2DDR		_IO(DMA_IOC_MAGIC, 2)
#define DMA_DDR2DDR		_IO(DMA_IOC_MAGIC, 3)

enum dma_type {
	AXI_DMA = 0,
	AXI_DMAR,
	PCIE_DMA,
	NONE_DMA,
};

struct dma_memcpy_test_data {
	void **srcs;
	void **dsts;
	int  *sizes;
	int  cnt;
	int  dma_type;
};

struct dma_memcpy_test_data *data;
int cmd = DMA_DDR2SYS;

void data_init()
{
	data = malloc(sizeof(struct dma_memcpy_test_data));
	data->srcs = NULL;
	data->dsts = NULL;
}

int get_param(int argc, char *argv[])
{
	int i = 0, cnt = 0, size = 0;
	char *type = NULL, *temp = NULL;
	for(i = 1; i < argc; i++)
	{
		if(!strncmp(argv[i],"dma_type", 8))
		{
			strtok(argv[i],"=");
			type = strtok(NULL,"=");
			if(!strcmp(type, "AXI"))
			{
				data->dma_type = AXI_DMA;
			}
			else if(!strcmp(type,"PCIE"))
			{
				data->dma_type = PCIE_DMA;
			}
			else
			{
				printf("error param of dma_type\n");
				return 1;
			}
		}

		if(!strncmp(argv[i],"dma_cnt", 7))
		{
			strtok(argv[i],"=");
			temp = strtok(NULL,"=");
			cnt = atoi(temp);
			if(!cnt)
			{
				printf("error param of dma_cnt\n");
				return 1;
			}
			data->cnt = cnt;
		}

		if(!strncmp(argv[i],"dma_size", 8))
		{
			strtok(argv[i],"=");
			temp = strtok(NULL,"=");
			size = atoi(temp);
			if(!size)
			{
				printf("error param of dma_size\n");
				return 1;
			}
		}

		if(!strncmp(argv[i],"dma_trans",9))
		{
			strtok(argv[i],"=");
			temp = strtok(NULL,"=");
			if(!strncmp(temp, "ddr2sys", 7))
			{
				cmd = DMA_DDR2SYS;
			}
			else if(!strncmp(temp, "ddr2ddr", 7))
			{
				cmd = DMA_DDR2DDR;
			}
			else if(!strncmp(temp, "sys2ddr", 7))
			{
				cmd = DMA_SYS2DDR;
			}
			else
			{
				printf("error param of trans_type\n");
				return -1;
			}
		}
	}

	if (type == NULL)
	{
		data->dma_type = AXI_DMA;
	}

	if(!cnt)
	{
		data->cnt = 5;
	}
	data->sizes = malloc(sizeof(int) * data->cnt);

	if(!size)
	{
		size = 4096;
	}

	for (i = 0;i < data->cnt; i++)
	{
		data->sizes[i] = size;
	}
	return 0;
}

int main(int argc,char *argv[])
{
	int fd = 0, i = 0,j = 0;
	char buf[GTEST_DMA_CDEV_SIZE];
	data_init();
	get_param(argc, argv);

	fd = open(GTEST_DMA_DEV, O_RDWR);
	if(fd < 0)
	{
		perror("open gtest_dma_test_dev failed");
		return -1;
	}

		switch (cmd)
		{
		case DMA_DDR2SYS:
			if(ioctl(fd, DMA_DDR2SYS, data) < 0)
			{
				perror("ioctl failed about DDR2SYS");
				return -1;
			}
			break;
		case DMA_DDR2DDR:
			if(PCIE_DMA == data->dma_type)
			{
				printf("\t\tPCIE传输不支持ddr2ddr!!!\n");
				break;
			}
			if(ioctl(fd, DMA_DDR2DDR, data) < 0)
			{
				perror("ioctl failed about DDR2SYS");
				return -1;
			}
			break;
		case DMA_SYS2DDR:
			if(ioctl(fd, DMA_SYS2DDR, data) < 0)
			{
				perror("ioctl failed about DDR2SYS");
				return -1;
			}
			break;
		default:
			break;
		}

		read(fd, buf, GTEST_DMA_CDEV_SIZE);
		puts(buf);
	free(data->sizes);
	free(data);
	return 0;
}
